﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using System.ComponentModel;
using Infragistics.Controls.Maps;
using Infragistics.Windows;
using System.Diagnostics;
using Infragistics.DragDrop;

namespace NetworkNode_BrindNodeIntoView
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        Canvas connectionHolder;
        bool isStartFixed = false;
        Path p = new Path();
        public MainWindow()
        {
            InitializeComponent();
            DataContext = new SampleTestData();
            xnn.DefineConnectionCallback = ConditionalFormatting;
            connectionHolder = new Canvas();

            p.Stroke = Brushes.Gray;
            p.StrokeThickness = 2;
            
        }


        private void listBox1_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            xnn.SelectedNodes.Clear();
            xnn.SelectedNodes.Add(xnn.Search((NodeModel item) => item.Label.StartsWith((listBox1.SelectedItem as NodeModel).Label)).FirstOrDefault());
            xnn.ZoomSelectedNodes();
            xnn.ZoomLevel = 2;
        }

        private Path ConditionalFormatting(NetworkNodeConnection connection, Point sp, Point tp, Point osp, Point otp)
        {
           
            connection.NetworkNode.LineEndCap = LineCap.SolidArrow;
            ContextMenu contextMenu = new ContextMenu();
            contextMenu.DataContext = connection;
            MenuItem item1 = new MenuItem { Header = "Info" };
            item1.Click += (s, e) =>
                {
                    NetworkNodeConnection con = (s as MenuItem).DataContext as NetworkNodeConnection;
                    MessageBox.Show(
                        "Connection between " +
                        (con.Source.Data as NodeModel).Label +
                        " and " +
                        (con.Target.Data as NodeModel).Label);
                };
            contextMenu.Items.Add(item1);
            MenuItem item2 = new MenuItem { Header = "Delete" };
            item2.Click += (s, e) =>
            {
                NetworkNodeConnection con = (s as MenuItem).DataContext as NetworkNodeConnection;

                (con.Source.Data as NodeModel).Connections.Remove(con.Data as NetworkConnection);
            };
            contextMenu.Items.Add(item2);


            Path path = new Path()
            {
                Data = new LineGeometry(tp, sp),
                Stroke = Brushes.DarkGray,
                StrokeThickness = 5,
                StrokeEndLineCap = PenLineCap.Triangle,
                StrokeStartLineCap = PenLineCap.Triangle,
                ContextMenu = contextMenu,
                DataContext = connection
            };
            path.MouseLeftButtonDown +=(ss,ee)=>
                {
                    ee.Handled = true;
                };

            DragSource drag = new DragSource 
            { 
                IsDraggable = true, 
                DragChannels = new ObservableCollection<string> { "delete" }
            };

            drag.DragStart += (ss, ee) =>
                {
                    ee.DragSource.Opacity = 0;
                    p.Opacity = 1;

                    double point = Math.Abs(Mouse.GetPosition(ee.DragSource).Y - ((ee.DragSource as Path).Data as LineGeometry).StartPoint.Y);
                    double len = Math.Abs(((ee.DragSource as Path).Data as LineGeometry).StartPoint.Y - ((ee.DragSource as Path).Data as LineGeometry).EndPoint.Y);

                    if (point < (len / 2))
                    {
                        p.Data = new LineGeometry(((ee.DragSource as Path).Data as LineGeometry).EndPoint, ((ee.DragSource as Path).Data as LineGeometry).EndPoint);
                        isStartFixed = false;
                    }
                    else
                    {
                        p.Data = new LineGeometry(((ee.DragSource as Path).Data as LineGeometry).StartPoint, ((ee.DragSource as Path).Data as LineGeometry).StartPoint);
                        isStartFixed = true;
                    }

                };
            drag.DragOver += (ss, ee) =>
                {
                    if (isStartFixed)
                        (p.Data as LineGeometry).EndPoint = Mouse.GetPosition(connectionHolder);
                    else
                        (p.Data as LineGeometry).StartPoint = Mouse.GetPosition(connectionHolder);
                };

            drag.Drop += (ss, ee) =>
                {
                    NetworkNodeConnection con = (ee.DragSource as Path).DataContext as NetworkNodeConnection;
                    (con.Source.Data as NodeModel).Connections.Remove(con.Data as NetworkConnection);
                    p.Opacity = 0;
                };

            DragDropManager.SetDragSource(path, drag);

            return path;

        }

        private void xnn_Loaded(object sender, RoutedEventArgs e)
        {
            (Utilities.GetDescendantFromType(sender as DependencyObject, typeof(Grid), false) as Grid).Children.Add(connectionHolder);
            connectionHolder.Children.Add(p);
        }
    }

    public class SampleTestData
    {
        public ObservableCollection<NodeModel> Nodes { get; set; }

        private const int K = 3;            // number of connections per node (maximum)
        private const int NUM_NODES = 15;   // number of nodes in the graph

        public SampleTestData()
        {
            Nodes = new ObservableCollection<NodeModel>();

            // Add NUM_NODES node objects to the collection
            for (int i = 0; i < NUM_NODES; i++)
            {
                NodeModel node = new NodeModel();
                node.Label = i.ToString();
                node.ToolTip = "ToolTip for " + node.Label;
                Nodes.Add(node);
            }

            int root = 0;
            int first = 1;
            int last = K;
            while (first < Nodes.Count)
            {
                Nodes[root].Connections = new ObservableCollection<NetworkConnection>();
                for (int i = first; i <= last; i++)
                {
                    if (i >= Nodes.Count)
                    {
                        break;
                    }
                    Nodes[root].Connections.Add(new NetworkConnection { Target = Nodes[i] });
                }
                root++;
                first = last + 1;
                last += K;
            }
        }
    }

    public class NodeModel : ObservableModel
    {
        private string _label;
        public string Label
        {
            get { return _label; }
            set
            {
                if (value != _label)
                {
                    _label = value;
                    OnPropertyChanged("Label");
                }
            }
        }

        private string _toolTip;
        public string ToolTip
        {
            get { return _toolTip; }
            set
            {
                if (value != _toolTip)
                {
                    _toolTip = value;
                    OnPropertyChanged("ToolTip");
                }
            }
        }

        private string _description;
        public string Description
        {
            get { return _description; }
            set
            {
                if (value != _description)
                {
                    _description = value;
                    OnPropertyChanged("Description");
                }
            }
        }

        private string _imagePath;
        public string ImagePath
        {
            get { return _imagePath; }
            set
            {
                if (value != _imagePath)
                {
                    _imagePath = value;
                    OnPropertyChanged("ImagePath");
                }
            }
        }

        public enum NodeModelType { Default, Attendee, Speaker, Company, Convention };
        private NodeModelType _nodeType;
        public NodeModelType NodeType
        {
            get { return _nodeType; }
            set
            {
                if (value != _nodeType)
                {
                    _nodeType = value;
                    OnPropertyChanged("NodeType");
                }
            }
        }

        private ObservableCollection<NetworkConnection> _connections;
        public ObservableCollection<NetworkConnection> Connections
        {
            get { return _connections; }
            set
            {
                if (value != _connections)
                {
                    _connections = value;
                    OnPropertyChanged("Connections");
                }
            }
        }

    }

    public class NetworkConnection : ObservableModel
    {
        private int _weight;
        public int Weight
        {
            get { return _weight; }
            set
            {
                if (value != _weight)
                {
                    _weight = value;
                    OnPropertyChanged("Weight");
                }
            }
        }

        private NodeModel _target;
        public NodeModel Target
        {
            get { return _target; }
            set
            {
                if (value != _target)
                {
                    _target = value;
                    OnPropertyChanged("Target");
                }
            }
        }

    }

    public class ObservableModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        public void OnPropertyChanged(string name)
        {
            if (PropertyChanged != null)
                PropertyChanged(this, new PropertyChangedEventArgs(name));
        }
    }
}
